/*  pulsard - monitor MGE Pulsar UPS
 * 
 *  Copyright (C) 2000 Philippe Marzouk <philm@users.sourceforge.net>
 *  All rights reserved.
 *
 */

/*
 *		       GNU GENERAL PUBLIC LICENSE
 *			  Version 2, June 1991
 *
 *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
 *			     675 Mass Ave, Cambridge, MA 02139, USA
 *  Everyone is permitted to copy and distribute verbatim copies
 *  of this license document, but changing it is not allowed.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <gpio/gpio.h>
#include <syslog.h>
#include <string.h>
#include <linux/limits.h>

#include "pulsard.h"
#include "options.h"
#include "serial.h"

/* global variables */
gpio_device *gdev;
gpio_device_settings settings;
char lockfile[PATH_MAX];

/*
* Initialize the communication with the UPS
*
*	return 0 on success, -1 on failure
*/
int serial_init()
{
	FILE *fp;
	int i,pid;
	char temp[PATH_MAX];
	char portfile[NAME_MAX];


	gpio_init();
	gdev = gpio_new(GPIO_DEVICE_SERIAL);

	serial_set_settings();

	if ( gpio_open(gdev) == GPIO_ERROR) {
		syslog(LOG_ERR, "Error opening device");
		return -1;
	}


	/*FIXME: ttyS2 hardcoded */
	strcpy(temp,port);
	if (strrchr(temp,'/') != NULL) {
		i = strrchr(temp,'/') - temp;
		strcpy(portfile,temp+i+1);
	}
	sprintf (lockfile, "%s/LCK..%s", LOCKDIR,portfile);
	if ((fp=fopen(lockfile,"r")) != NULL) {
		fscanf(fp,"%u",&pid);
		syslog(LOG_CRIT,"Port already used by process %i or stale lock file - exiting",pid);
		printf("Port already used by process %i or stale lock file - exiting\n",pid);
		fclose(fp);
		return -1;
		
	}
	if (kill_ups_power == 0) { /* we don't want a lock file in this case */
		if ((fp=fopen(lockfile,"w")) != NULL) {
			fprintf(fp,"%u",getpid());
			fclose(fp);
		}
		else {
			syslog(LOG_CRIT,"Error creating lockfile %s - exiting",lockfile);
			return -1;
		}
	}

	gpio_set_timeout(gdev,7000);
	
	return 0;
}

int serial_set_settings()
{

 	strcpy(settings.serial.port, port);   

    settings.serial.speed = 2400;
    settings.serial.bits = 8;
    settings.serial.parity = 0;
    settings.serial.stopbits = 1;

	return gpio_set_settings(gdev, settings);
}

int serial_set_pin()
{
	if (gpio_set_pin(gdev,PIN_RTS,1) == GPIO_ERROR) {
		syslog(LOG_ERR, "Error setting RTS line");
		return -1;
	}
}

/*
* Close the serial dev
*
*/
int serial_close()
{
	FILE *fp;
	int pid;

	gpio_close(gdev);
	gpio_free(gdev);
	if ((fp=fopen(lockfile,"r")) != NULL) {
		fscanf(fp,"%u",&pid);
		fclose(fp);
	}
	if(pid==getpid()) { /* We don't want to remove the lock file if 
								we did'nt set it */
		if(unlink(lockfile)!=0)
			syslog(LOG_DEBUG,"error removing lockfile");
	}
	
	return 0;
}


/*
* Sends the given data of the given length to the UPS
*
* buf	the data
* len	the length
*
*/
int serial_send(char *buf, int len)
{

	if(gpio_write(gdev,buf,len) == GPIO_ERROR) {
		syslog(LOG_ERR,"Error while sending");
		return -1;
	}
/* 	dump_hex("sent",buf,len);  */
/*	syslog(LOG_DEBUG,"sent: %s",buf); */

	return 0;
}

/*
*  read data one byte at a time
*
* 
*/
int serial_read()
{
	static unsigned char cache[512];
	static unsigned char *cachep = cache;
	static unsigned char *cachee = cache;
	int recv;

/* if still data in cache, get it */
	if (cachep < cachee)
	{
		return (int) *cachep++;
	}


	recv = gpio_read(gdev, cache, 1);

	if (recv == GPIO_ERROR || recv == GPIO_TIMEOUT)
		return -1;
	cachep = cache;
	cachee = cache + recv;

	if (recv)
	{
		return (int) *cachep++;
	}

	return -1;
}
